# Copyright (c) 2017 3D Control Systems LTD

# 3DPrinterOS client is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# 3DPrinterOS client is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with 3DPrinterOS client.  If not, see <http://www.gnu.org/licenses/>.

# Author: Maksym Kozlov

import re
import time

import logging


class BlockingHeatingEmulator:
    MAX_TIME_LIMIT = 30
    MAX_DELTA_TIME_LIMIT = 15
    DELTA_TEMP = 0.1

    def __init__(self, sender):
        self.sender = sender
        self.enabled = False
        self.original_gcode = None
        self.tool = None
        self.last_change_time = None
        self.max_temp = 0
        self.logger = logging.getLogger(self.__class__.__name__)

    def start(self, line):
        if self.enabled:
            return
        if 'M109' in line:
            tool_match = re.match('.+T(\d+)', line)
            if tool_match:
                tool = int(tool_match.group(1)) + 1
            else:
                tool = 1
            temp_match = re.match('.+S([\d\.]+)', line)
            if temp_match:
                if tool < len(self.sender.target_temps) and float(temp_match.group(1)) > 0:
                    self.__enable(line, tool)
                    return line.replace('M109', 'M104')
        elif 'M190' in line:
            temp_match = re.match('.+S([\d\.]+)', line)
            if temp_match and float(temp_match.group(1)) > 0:
                self.__enable(line, 0)
                return line.replace('M190', 'M140')

    def __enable(self, line, tool):
        self.original_gcode = line
        self.tool = tool
        self.last_change_time = time.time()
        self.max_temp = 0
        self.enabled = True

    def disable(self, send_original_gcode=True):
        if self.original_gcode and send_original_gcode:
            self.sender.send_now(self.original_gcode)
        self.original_gcode = None
        self.enabled = False

    def check_temperatures(self):
        if not self.enabled:
            return
        t = time.time()
        if t - self.last_change_time > self.MAX_TIME_LIMIT:
            return self.disable()
        target = self.sender.target_temps[self.tool]
        if target <= 0:
            return
        temp = self.sender.temps[self.tool]
        delta = target*self.DELTA_TEMP
        if temp >= target or (temp >= target-delta and t-self.last_change_time > self.MAX_DELTA_TIME_LIMIT):
            return self.disable()
        if temp > self.max_temp:
            self.max_temp = temp
            self.last_change_time = t
